@@ -2,7 +2,8 @@
 Modify sequence file(s) in place.
 """
 
-import convert
+from seqmagick.subcommands import convert
+import argparse
 import logging
 import os.path
 import shutil
@@ -14,8 +15,10 @@ def build_parser(parser):
     """
     convert.add_options(parser)
 
-    parser.add_argument('input_files', metavar="sequence_file", nargs='+',
-            help="Sequence file(s) to mogrify")
+    parser.add_argument(
+        'input_files', metavar="sequence_file", nargs='+',
+        type=argparse.FileType('r+'),
+        help="Sequence file(s) to mogrify")
 
     return parser
 
@@ -27,13 +30,11 @@ def action(arguments):
     """
     for input_file in arguments.input_files:
         logging.info(input_file)
-        bn = os.path.basename(input_file)
+        bn = os.path.basename(input_file.name)
         # Generate a temporary file
         with tempfile.NamedTemporaryFile(prefix='smagick', suffix=bn,
                 delete=False) as tf:
-            temp_name = tf.name
-
-        convert.transform_file(input_file, temp_name, arguments)
+            convert.transform_file(input_file, tf, arguments)
         # Overwrite the original file
-        shutil.move(temp_name, input_file)
+        shutil.move(tf.name, input_file.name)
 
