@@ -91,28 +91,55 @@ def first_name_capture(records):
             yield record
 
 
-def cut_sequences(records, start, end):
+def include_from_file(records, handle):
     """
-    Cut sequences given a one-based range.  Includes last item.
+    Filter the records, keeping only sequences whose ID is contained in the
+    handle.
     """
-    start = start - 1
+    ids = set(i.strip() for i in handle)
+
     for record in records:
-        cut_record = SeqRecord(record.seq[start:end], id=record.id,
-                               name=record.name, description=record.description)
+        if record.id.strip() in ids:
+            yield record
 
-        # Copy the annotations over
-        for k, v in record.annotations.items():
-            # Trim if appropriate
-            if isinstance(v, (tuple, list)) and len(v) == len(record):
-                v = v[start:end]
-            cut_record.annotations[k] = v
 
-        # Letter annotations must be lists / tuples / strings of the same
-        # length as the sequence
-        for k, v in record.letter_annotations.items():
-            cut_record.letter_annotations[k] = v[start:end]
+def exclude_from_file(records, handle):
+    """
+    Filter the records, keeping only sequences whose ID is not contained in the
+    handle.
+    """
+    ids = set(i.strip() for i in handle)
 
-        yield cut_record
+    for record in records:
+        if record.id.strip() not in ids:
+            yield record
+
+
+def isolate_region(sequences, start, end, gap_char='-'):
+    """
+    Replace regions before and after start:end with gap chars
+    """
+    # Check arguments
+    if end <= start:
+        raise ValueError("start of slice must precede end ({0} !> {1})".format(
+            end, start))
+
+    for sequence in sequences:
+        seq = sequence.seq
+        start_gap = gap_char * start
+        end_gap = gap_char * (len(seq) - end)
+        seq = Seq(start_gap + str(seq[start:end]) + end_gap,
+                alphabet=seq.alphabet)
+        sequence.seq = seq
+        yield sequence
+
+
+def cut_sequences(records, cut_slice):
+    """
+    Cut sequences given a slice.
+    """
+    for record in records:
+        yield record[cut_slice]
 
 
 def lower_sequences(records):
@@ -281,7 +308,6 @@ def seq_include(records, filter_regex):
             yield record
 
 
-
 def seq_exclude(records, filter_regex):
     """
     Filter any sequences who's seq matches the filter. Ignore case.
@@ -317,26 +343,50 @@ def tail(records, tail, record_count):
             yield record
 
 
-def squeeze(records, gaps):
+# Squeeze-related
+def gap_proportion(sequences, gap_chars='-'):
     """
-    Remove any gaps that are present in the same position across all sequences in an alignment.
+    Generates a list with the proportion of gaps by index in a set of
+    sequences.
     """
-    logging.info('Applying _squeeze generator: '
+    aln_len = None
+    gaps = []
+    for i, sequence in enumerate(sequences):
+        if aln_len is None:
+            aln_len = len(sequence)
+            gaps = [0] * aln_len
+        else:
+            if not len(sequence) == aln_len:
+                raise ValueError(("Unexpected sequence length {0}. Is this "
+                                  "an alignment?").format(len(sequence)))
+
+        # Update any gap positions in gap list
+        for j, char in enumerate(sequence.seq):
+            if char in gap_chars:
+                gaps[j] += 1
+
+    sequence_count = float(i + 1)
+    gap_props = [i / sequence_count for i in gaps]
+    return gap_props
+
+
+def squeeze(records, gap_threshold, sequence_iterator):
+    """
+    Remove any gaps that are present in the same position across all sequences
+    in an alignment.  Takes a second sequence iterator for determining gap
+    positions.
+    """
+    logging.info('Applying squeeze: '
                  'removing any gaps that are present '
                  'in the same position across all sequences in an alignment.')
-    sequence_length = len(gaps)
+
+    gap_proportions = gap_proportion(sequence_iterator)
+    keep_columns = [g < gap_threshold for g in gap_proportions]
+
     for record in records:
-        sequence = list(str(record.seq))
-        if len(sequence) != sequence_length:
-            raise ValueError("Unexpected sequence length: {0} != {1} "
-                    "Is this an alignment?".format(len(sequence),
-                                                   sequence_length))
-        squeezed = []
-        position = 0
-        while (position < sequence_length):
-            if bool(gaps[position]) is False:
-                squeezed.append(sequence[position])
-            position += 1
+        sequence = str(record.seq)
+        # Trim
+        squeezed = itertools.compress(sequence, keep_columns)
         yield SeqRecord(Seq(''.join(squeezed)), id=record.id,
                         description=record.description)
 
@@ -445,29 +495,15 @@ def min_length_discard(records, min_length):
             yield record
 
 
-# Begin squeeze-related functions
-
-def _is_gap(character):
+def min_ungap_length_discard(records, min_length):
     """
+    Discard any records that are shorter than min_length after removing gaps.
     """
-    if character == '-':
-        return 1
-    else:
-        return 0
+    for record in records:
+        if len(record.seq.ungap('-')) >= min_length:
+            yield record
 
 
-def gap_check(gap, character):
-    """
-    Build up a gaps list that is used on all sequences
-    in an alignment.
-    """
-    # Skip any characters that have already been found
-    if gap == 0:
-        return gap
-    return int(bool(gap) & bool(_is_gap(character)))
-
-# End squeeze-related functions
-
 def sort_length(source_file, source_file_type, direction=1):
     """
     Sort sequences by length. 1 is ascending (default) and 0 is descending.
@@ -487,7 +523,7 @@ def sort_length(source_file, source_file_type, direction=1):
     else:
         ids = [seq_id for (length, seq_id) in len_and_ids]
     del len_and_ids #free this memory
-    record_index = SeqIO.index(source_file, source_file_type)
+    record_index = SeqIO.index(source_file.name, source_file_type)
     records = (record_index[seq_id] for seq_id in ids)
 
     return records
@@ -509,7 +545,7 @@ def sort_name(source_file, source_file_type, direction=1):
 
     if direction == 0:
         ids = reversed(ids)
-    record_index = SeqIO.index(source_file, source_file_type)
+    record_index = SeqIO.index(source_file.name, source_file_type)
     records = (record_index[id] for id in ids)
 
     return records
